@@ -1,5 +1,26 @@
-#ifndef _dconf_engine_h_
-#define _dconf_engine_h_
+/*
+ * Copyright © 2010 Codethink Limited
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the licence, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Author: Ryan Lortie <desrt@desrt.ca>
+ */
+
+#ifndef __dconf_engine_h__
+#define __dconf_engine_h__
 
 #include <dconf-readtype.h>
 #include <dconf-resetlist.h>
@@ -7,59 +28,94 @@
 
 typedef struct _DConfEngine DConfEngine;
 
+/**
+ * DConfEngineMessage:
+ *
+ * This structure represents a number of DBus method call messages that #DConfEngine would like to send.
+ *
+ * #DConfEngine itself is unaware of a particular DBus or main loop implementation.  As such, all requests are
+ * synchronous and non-blocking, but most of them produce a #DConfEngineMessage describing messages that must be
+ * sent in order for the operation to be completed.
+ *
+ * @bus_name, @object_path, @interface_name, @method_name specify the respective header fields of the method
+ * call.  These are always equal for all of the calls contained within a single #DConfEngineMessage.
+ *
+ * @reply_type is the expected reply type of the method call.  This is also the same for all calls contained
+ * within a single #DConfEngineMessage.
+ *
+ * @n_messages is the number of messages to send.
+ *
+ * @bus_types and @parameters are both arrays, of length @n_messages.  Each element of @bus_type is the bus type
+ * to send each method call on and each of @parameters is the body of that call.  The reason that there may be
+ * several messages is that a single dconf "watch" operation may need to send multiple DBus "AddMatch" calls
+ * (and usually to multiple busses).
+ *
+ * Each element in @bus_types is either 'y' for system bus or 'e' for session bus.
+ *
+ * A #DConfEngineMessage is always stack-allocated by the caller.  It must be cleared using
+ * dconf_engine_message_destroy() when done.  It may be copied using dconf_engine_message_copy().
+ */
 typedef struct
 {
-  gint         bus_type;
-  const gchar *destination;
-  const gchar *object_path;
-  const gchar *interface;
-  const gchar *method;
-  const GVariantType *reply_type;
-  GVariant    *body;
+  const gchar         *bus_name;
+  const gchar         *object_path;
+  const gchar         *interface_name;
+  const gchar         *method_name;
+
+  gint                 n_messages;
+  GVariant           **parameters;
+  const gchar         *bus_types;
+
+  const GVariantType  *reply_type;
 } DConfEngineMessage;
 
+typedef GVariant *    (*DConfEngineServiceFunc)                         (DConfEngineMessage      *message);
+void                    dconf_engine_message_copy                       (DConfEngineMessage      *orig,
+                                                                         DConfEngineMessage      *copy);
+void                    dconf_engine_message_destroy                    (DConfEngineMessage      *message);
 
+void                    dconf_engine_set_service_func                   (DConfEngineServiceFunc   func);
+DConfEngine *           dconf_engine_new                                (const gchar             *profile);
+DConfEngine *           dconf_engine_new_for_db                         (DConfEngineServiceFunc  *service,
+                                                                         const gchar             *db_name);
+guint64                 dconf_engine_get_state                          (DConfEngine             *engine);
 
-typedef GVariant *    (*DConfEngineServiceFunc)                         (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message);
-
-DConfEngine *           dconf_engine_new                                (const gchar             *context);
-void                    dconf_engine_unref                              (DConfEngine             *engine);
-DConfEngine *           dconf_engine_ref                                (DConfEngine             *engine);
+void                    dconf_engine_free                               (DConfEngine             *engine);
 
 GVariant *              dconf_engine_read                               (DConfEngine             *engine,
-                                                                         const gchar             *key,
-                                                                         DConfReadType            type);
+                                                                         const gchar             *key);
+GVariant *              dconf_engine_read_default                       (DConfEngine             *engine,
+                                                                         const gchar             *key);
+GVariant *              dconf_engine_read_no_default                    (DConfEngine             *engine,
+                                                                         const gchar             *key);
 gchar **                dconf_engine_list                               (DConfEngine             *engine,
                                                                          const gchar             *path,
                                                                          DConfResetList          *resets,
-                                                                         gsize                   *length);
+                                                                         gint                    *length);
 
 void                    dconf_engine_get_service_info                   (DConfEngine             *engine,
                                                                          const gchar            **bus_type,
                                                                          const gchar            **destination,
                                                                          const gchar            **object_path);
 gboolean                dconf_engine_is_writable                        (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message,
-                                                                         const gchar             *name,
-                                                                         GError                 **error);
+                                                                         const gchar             *name);
 gboolean                dconf_engine_write                              (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message,
                                                                          const gchar             *key,
                                                                          GVariant                *value,
+                                                                         DConfEngineMessage      *message,
                                                                          GError                 **error);
 gboolean                dconf_engine_write_many                         (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message,
                                                                          const gchar             *prefix,
                                                                          const gchar * const     *keys,
                                                                          GVariant               **values,
+                                                                         DConfEngineMessage      *message,
                                                                          GError                 **error);
 void                    dconf_engine_watch                              (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message,
-                                                                         const gchar             *name);
+                                                                         const gchar             *name,
+                                                                         DConfEngineMessage      *message);
 void                    dconf_engine_unwatch                            (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message,
-                                                                         const gchar             *name);
+                                                                         const gchar             *name,
+                                                                         DConfEngineMessage      *message);
 gboolean                dconf_engine_decode_notify                      (DConfEngine             *engine,
                                                                          const gchar             *anti_expose,
                                                                          const gchar            **prefix,
@@ -70,9 +126,9 @@ gboolean                dconf_engine_decode_notify                      (DConfEn
                                                                          const gchar             *member,
                                                                          GVariant                *body);
 void                    dconf_engine_set_locked                         (DConfEngine             *engine,
-                                                                         DConfEngineMessage      *message,
                                                                          const gchar             *path,
-                                                                         gboolean                 locked);
+                                                                         gboolean                 locked,
+                                                                         DConfEngineMessage      *message);
 
 gboolean                dconf_engine_interpret_reply                    (DConfEngineMessage      *message,
                                                                          const gchar             *sender,
@@ -80,4 +136,4 @@ gboolean                dconf_engine_interpret_reply                    (DConfEn
                                                                          gchar                  **tag,
                                                                          GError                 **error);
 
-#endif /* _dconf_engine_h_ */
+#endif /* __dconf_engine_h__ */
